#!/usr/bin/env python

# Generic Dummy Stage is meant to test a candidate framework or harness for interface compliance
# and robustness. It can be configured to look & feel to the outside like a typical processing stage
# taking as inputs and outputs file pointers to memory maps. Instead of doing a stage's real work,
# though, it performs some dummy operation (default: for one spectral input, one float ref and one 
# spectral output, it adds the ref value to each input spectral point and writes the result to the
# spectral output.

# Author: Maciek Smuga-Otto, UW SSEC
# Copyright 2004, University of Wisconsin, Madison


from mmap import mmap, MAP_SHARED, PROT_READ, PROT_WRITE
from sys import exit, argv, stdin, stdout, stderr
from os import environ
from struct import pack, unpack, calcsize

CVS_ID = '$Id: ResamplingDummyStage.py,v 1.1 2004/03/03 19:27:58 maciek Exp $'
DOUBLE_SIZE = 8 # Each spectrum value is a double
REF_SIZE = calcsize('ddLLL') # packed format for reference database is ddLLL

def read_mapinfo( input_string ):
    """ processes input formatted as a comma-separated tuple of ints. """
    return [int(i) for i in input_string.split(',')]

class DummyPlug:
    def __init__(self):
        # input
        self.infd, self.insize, self.inlen = read_mapinfo( environ['varInputSpectra'] )
        print >> stderr, "inputs - fd: %s size: %s len: %s" % (self.infd, self.insize, self.inlen) #DEBUG
        self.infp = mmap( self.infd, self.insize * self.inlen * DOUBLE_SIZE, MAP_SHARED, PROT_READ )
    
        # output
        self.outfd, self.outsize, self.outlen = read_mapinfo( environ['varOutputSpectra'] )
        print >> stderr, "outputs - fd: %s size: %s len: %s" % (self.outfd, self.outsize, self.outlen) #DEBUG
        self.outfp = mmap( self.outfd, self.outsize * self.outlen * DOUBLE_SIZE )
    
        # reference
        self.reffd, self.reflen = read_mapinfo( environ['refSamplingSettings'] )
        print >> stderr, "reference - fd: %s len: %s" % (self.reffd, self.reflen) #DEBUG
        self.reffp = mmap( self.reffd, self.reflen * REF_SIZE, MAP_SHARED, PROT_READ )
    
    def __call__(self, in_idx, out_idx, ref_idx): #FIXME - is this the correct argument order?
        # print >> stderr, "%s is type %s" % (in_idx * self.insize * DOUBLE_SIZE, type(in_idx * self.insize * DOUBLE_SIZE)) #DEBUG
        self.infp.seek( in_idx * self.insize * DOUBLE_SIZE ) #FIXME - do I need DOUBLE_SIZE here?
        self.outfp.seek( out_idx * self.outsize * DOUBLE_SIZE ) # ditto.
        self.reffp.seek( ref_idx * REF_SIZE )
        
        # Check reference DB here for compliance with standard!
        ref = unpack( 'ddLLL', self.reffp.read( REF_SIZE ) )
        # print >> stderr, "ref[%s]: %s:%s:%s:%s:%s" % ( ref_idx,) + ref #DEBUG
        
        for i in range( self.insize ):
            in_pt = unpack('d', self.infp.read( DOUBLE_SIZE ))[0]
            #print >> stderr, in_pt #DEBUG
            self.outfp.write( pack('d', in_pt + 1 ) )
        
def main():
    dummy = DummyPlug()
    
    while (1):
        args = raw_input('').split()
        cmd = args.pop(0)
        # print >> stderr, "cmd: %s args: %s" % (cmd, args) #DEBUG
        
        if cmd == 'ping':
            print '0 OK, ready for command'
        elif cmd == 'ident':
            print CVS_ID
        elif cmd == 'resample':
            args = [int(a) for a in args]
            dummy( *args )
            print '0 OK, ready for command'
        elif cmd == 'exit':
            print 'EXITING'
            exit(0)
        else:
            print 'INVALID COMMAND'
        stdout.flush()


if __name__ == '__main__':
    main()