#!/usr/bin/env python
"""
$Id: simpletest0.py,v 1.5 2004/03/03 05:11:00 rayg Exp $

Simplified test for ResamplingStage executable.

Author: R.K.Garcia
"""


from mmap import *
from struct import *
from math import sin

from os import tempnam, fork, environ, execv, dup2, fdopen, pipe, waitpid


WIDTH=17


def createRefTable( ):
    data = pack( 'ddLLL', 1.0, 1.1, WIDTH, 0, 0 )  
    fpref = file( tempnam(), 'wb+' )
    fpref.write( data )
    fpref.seek(0)
    return fpref
    
def createInputData( ):
    L = [ sin(float(x)) for x in range(0,WIDTH) ]
    data = pack( '%dd' % WIDTH, *L )
    fpin = file( tempnam(), 'wb+' )
    fpin.write( data )
    fpin.seek(0)
    return fpin
    
def createOutputData():
    L = [ 0.0 ] * WIDTH
    data = pack( '%dd' % WIDTH, *L )
    fpout = file( tempnam(), 'wb+' )
    fpout.write( data ) 
    fpout.seek(0)
    return fpout
    
def invoke( (fpout,fpin,fpref), exe ):
    rfd,wfd = pipe()
    writable0 = fdopen( wfd, 'w' )
    readable0 = fdopen( rfd, 'r' )
    rfd,wfd = pipe()
    writable1 = fdopen( wfd, 'w' )
    readable1 = fdopen( rfd, 'r' )
    
    pid = fork( )
    if (pid==0): # then I am the child
        nenv = { 'varInputSpectra': '%d,%d,1' % (fpin.fileno(),WIDTH),
                    'varOutputSpectra': '%d,%d,1' % (fpout.fileno(),WIDTH),
                    'refSamplingSettings': '%d,1' % fpref.fileno() }
        environ.update( nenv )
        writable0.close()
        readable1.close()
        print "child executing %s" % exe
        # change stdin and stdout
        dup2( readable0.fileno(), 0 ) #stdin
        dup2( writable1.fileno(), 1 ) #stdout
        execv( exe, [ exe ] )
        # end .. 
        
    readable0.close()
    writable1.close()
    return writable0, readable1, pid
    
def command( (writable0,readable1,pid), cmd ):
    print >>writable0, cmd
    writable0.flush()
    return readable1.readline()
    
def ping( link ):
    return command( link, "ping" )
        
def ident( link ):
    return command( link, "ident" )
        
def resample( link ):
    return command( link, 'resample 0 0 0' )

def doexit( link ):
    return command( link, 'exit' )
    

def test1():
    print "creating input and reference data.."
    crd = createRefTable()
    cid = createInputData()
    cod = createOutputData()
    
    print "forking executable stage.." 
    wrp = invoke( (cod,cid,crd), './ResamplingStage' )
    print "ping result: %s" % ping( wrp ).strip()
    print "ident result: %s" % ident( wrp ).strip()
    print "resample result: %s" % resample( wrp )
    print "exit request result: %s" % doexit( wrp )
    print "waiting for exit.." 
    waitpid( wrp[2], 0 )
    print "done."
    
if __name__=='__main__':
    test1()
    
    
    
        
